import { CommonModule } from '@angular/common';
import { InjectionToken, EventEmitter, ElementRef, Renderer2, Inject, Input, Output, Directive, HostListener, Optional, ContentChildren, NgModule } from '@angular/core';
import { asyncScheduler, Subject, BehaviorSubject, Subscription, merge, combineLatest, ReplaySubject } from 'rxjs';
import { throttleTime, switchMap, map, startWith } from 'rxjs/operators';
import { NG_VALUE_ACCESSOR } from '@angular/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Maps an object of control states to a value given by the `mapFunc`.
 *
 * \@example
 * // Mapping of:
 * const controls = {
 *  firstControl: {
 *    value: 'firstValue',
 *    ...
 *  },
 *  secondControl: {
 *    value: 'secondValue',
 *    ...
 *  },
 * }
 *
 * // With the function:
 * const mapFunc = (control) => control.value
 *
 * // Will result in:
 * const result = {
 *  firstControl: 'firstValue',
 *  secondControl: 'secondValue'
 * }
 * @template TControls, R
 * @param {?} controls Object which contains keys and associated control update.
 * @param {?} mapFunc Mapper function to convert the update.
 *
 * @return {?}
 */
function mapFormGroupControlStates(controls, mapFunc) {
    return mapFormControls(controls, mapFunc);
}
/**
 * Maps an object of control summaries to a value given by the `mapFunc`.
 *
 * \@example
 * // Mapping of:
 * const controls = {
 *  firstControl: {
 *    value: 'firstValue',
 *    ...
 *  },
 *  secondControl: {
 *    value: 'secondValue',
 *    ...
 *  },
 * }
 *
 * // With the function:
 * const mapFunc = (control) => control.value
 *
 * // Will result in:
 * const result = {
 *  firstControl: 'firstValue',
 *  secondControl: 'secondValue'
 * }
 * @template TControls, R
 * @param {?} controls Object which contains keys and associated control summaries.
 * @param {?} mapFunc Mapper function to convert the update.
 *
 * @return {?}
 */
function mapFormGroupControlSummaries(controls, mapFunc) {
    return mapFormControls(controls, mapFunc);
}
/**
 * Maps an object of control updates to a value given by the `mapFunc`.
 *
 * \@example
 * // Mapping of:
 * const controls = {
 *  firstControl: {
 *    value: 'firstValue',
 *    ...
 *  },
 *  secondControl: {
 *    value: 'secondValue',
 *    ...
 *  },
 * }
 *
 * // With the function:
 * const mapFunc = (control) => control.value
 *
 * // Will result in:
 * const result = {
 *  firstControl: 'firstValue',
 *  secondControl: 'secondValue'
 * }
 * @template TControls, R
 * @param {?} controls Object which contains keys and associated control updates.
 * @param {?} mapFunc Mapper function to convert the update.
 *
 * @return {?}
 */
function mapFormGroupControlUpdates(controls, mapFunc) {
    return mapFormControls(controls, mapFunc);
}
/**
 * @template TControls, R
 * @param {?} controls
 * @param {?} mapFunc
 * @return {?}
 */
function mapFormControls(controls, mapFunc) {
    /** @type {?} */
    const result = Object.entries(controls)
        .map((/**
     * @param {?} __0
     * @return {?}
     */
    ([key, control]) => ({
        [key]: mapFunc(control, key),
    })))
        .reduce((/**
     * @param {?} ctrl1
     * @param {?} ctrl2
     * @return {?}
     */
    (ctrl1, ctrl2) => (Object.assign({}, ctrl1, ctrl2))), {});
    return (/** @type {?} */ (result));
}
/**
 * Converts an Angular validator for the usage with this framework.
 * Does **not** support asynchronous validators.
 *
 * Only a subset of properties is available for the Angular validators:
 * `value, dirty, pristine, touched, untouched, enabled, disabled`
 * @template T
 * @param {?} fn Angular validator function..
 * @return {?}
 */
function validatorOf(fn) {
    return (/**
     * @param {?} control
     * @return {?}
     */
    (control) => fn((/** @type {?} */ ({
        value: control.value,
        dirty: !control.pristine,
        pristine: control.pristine,
        touched: !control.untouched,
        untouched: control.untouched,
        enabled: !control.disabled,
        disabled: control.disabled,
    }))));
}
/**
 * Merges an array of `FormControlErrors`.
 * Returns null if all errors are null.
 * @param {...?} errors An array of `FormControlErrors` which will be merged into a single `FormControlErrors` value.
 * @return {?}
 */
function mergeFormControlErrors(...errors) {
    return errors.reduce((/**
     * @param {?} e1
     * @param {?} e2
     * @return {?}
     */
    (e1, e2) => {
        if (!e1 && !e2) {
            return null;
        }
        if (!e1) {
            return e2;
        }
        if (!e2) {
            return e1;
        }
        return Object.assign({}, e1, e2);
    }), null);
}
/**
 * Merges an array of errors.
 * The different child control errors will also be merged.
 * Returns null if all errors are null.
 * \@example
 * // Merge of:
 * {
 *   stringControl: {
 *     firstError: 'firstError'
 *   }
 * },
 * {
 *   stringControl: {
 *     secondError: 'secondError'
 *   }
 * }
 *
 * // Will result in:
 * {
 *   stringControl: {
 *     firstError: 'firstError',
 *     secondError: 'secondError'
 *   }
 * }
 *
 * @template TControls
 * @param {...?} errors Multiple errors to be merged.
 *
 * @return {?}
 */
function mergeFormGroupErrors(...errors) {
    return errors.reduce((/**
     * @param {?} group1
     * @param {?} group2
     * @return {?}
     */
    (group1, group2) => {
        if (!group1 && !group2) {
            return null;
        }
        if (!group1) {
            return group2;
        }
        if (!group2) {
            return group1;
        }
        return Object.assign({}, group1, group2, Object.keys(group1)
            .filter((/**
         * @param {?} key1
         * @return {?}
         */
        key1 => Object.keys(group2).find((/**
         * @param {?} key2
         * @return {?}
         */
        key2 => key1 === key2))))
            .map((/**
         * @param {?} key
         * @return {?}
         */
        key => ({
            [key]: mergeFormControlErrors(group1[key], group2[key]),
        })))
            .reduce((/**
         * @param {?} e1
         * @param {?} e2
         * @return {?}
         */
        (e1, e2) => (Object.assign({}, e1, e2))), {}));
    }), null);
}
/**
 * Merges an array of errors.
 * Errors at each index will also be merged.
 * Returns null if all errors are null.
 * \@example
 * // Merge of:
 * [
 *   {
 *     firstError: 'firstError'
 *   }
 * ],
 * [
 *   stringControl: {
 *     secondError: 'secondError'
 *   }
 * ]
 *
 * // Will result in:
 * [
 *   {
 *     firstError: 'firstError',
 *     secondError: 'secondError'
 *   }
 * ]
 *
 * @param {...?} errors Multiple errors to be merged.
 *
 * @return {?}
 */
function mergeFormArrayErrors(...errors) {
    /** @type {?} */
    const mergedErrors = errors
        .filter(Boolean)
        .reduce((/**
     * @param {?} arr1
     * @param {?} arr2
     * @return {?}
     */
    (arr1, arr2) => (arr1.length >= arr2.length ? arr1 : arr2)), [])
        .map((/**
     * @param {?} _
     * @param {?} i
     * @return {?}
     */
    (_, i) => mergeFormControlErrors(...errors.map((/**
     * @param {?} arrayErrors
     * @return {?}
     */
    arrayErrors => arrayErrors[i])))));
    return mergedErrors.filter(Boolean).length > 0 ? mergedErrors : null;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Returns a `FormControlState` which represents the input state with the applied update.
 * The returned value is a new constructed object, except when the update is invalid.
 *
 * @template T
 * @param {?} control The `FormControlState` which the update will be applied to.
 * @param {?} update The `FormControlUpdate` itself.
 * @return {?}
 */
function reduceFormControl(control, update) {
    if (!update) {
        return control;
    }
    return Object.assign({}, control, update);
}
/**
 * Returns a `FormGroupState` which represents the input state with the applied update.
 * The returned value is a new constructed object, except when the update is invalid.
 *
 * @template TControls
 * @param {?} group
 * @param {?} update The `FormGroupUpdate` itself.
 * @return {?}
 */
function reduceFormGroup(group, update) {
    if (!update) {
        return group;
    }
    return Object.assign({}, group, update, { controls: Object.assign({}, group.controls, (update.controls
            ? mapFormGroupControlUpdates(update.controls, (/**
             * @param {?} control
             * @param {?} key
             * @return {?}
             */
            (control, key) => (Object.assign({}, group.controls[key], control))))
            : {})) });
}
/**
 * Returns a `FormArrayState` which represents the input state with the applied update.
 * The returned value is a new constructed object, except when the update is invalid.
 *
 * @template T
 * @param {?} group
 * @param {?} update The `FormArrayUpdate` itself. Invalid values and empty objects inside the `update.controls` array will be ignored.
 * @return {?}
 */
function reduceFormArray(group, update) {
    if (!update) {
        return group;
    }
    return Object.assign({}, group, update, { controls: group.controls.map((/**
         * @param {?} control
         * @param {?} index
         * @return {?}
         */
        (control, index) => update && update.controls ? reduceFormControl(control, update.controls[index]) : control)) });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Initializes a new `FormControlState`.
 * @see `FormControlInit`
 *
 * \@example
 * initFormControl(['value', [validator]])
 * initFormControl({value: 'value', validators: [validator], disabled: true, ...})
 * @template T
 * @param {?} initial
 * @return {?}
 */
function initFormControl(initial) {
    return Array.isArray(initial)
        ? initFormControlFromTuple(initial)
        : initFormControlFromUpdate(initial);
}
/**
 * Initializes a new `FormGroupState`.
 * @see `FormGroupInit`
 *
 * \@example
 * initFormGroup({
 *  firstControl: ['firstValue', [validator]],
 *  secondControl: {value: 'value', validators: [validator], disabled: true, ...}
 * })
 * @template TControls
 * @param {?} controls An object containing control initializations.
 * @return {?}
 */
function initFormGroup(controls) {
    return {
        controls: (/** @type {?} */ (Object.entries(controls)
            .map((/**
         * @param {?} __0
         * @return {?}
         */
        ([key, initial]) => ({
            [key]: initFormControl(initial),
        })))
            .reduce((/**
         * @param {?} ctrl1
         * @param {?} ctrl2
         * @return {?}
         */
        (ctrl1, ctrl2) => (Object.assign({}, ctrl1, ctrl2))), {}))),
    };
}
/**
 * Initializes a new `FormArrayState`.
 * @see `FormArrayInit`
 *
 * \@example
 * initFormArray([
 *  ['firstValue', [validator]],
 *  {value: 'value', validators: [validator], disabled: true, ...}
 * ])
 * @template T
 * @param {?} initial
 * @return {?}
 */
function initFormArray(initial) {
    return {
        controls: initial.map((/**
         * @param {?} init
         * @return {?}
         */
        init => initFormControl(init))),
    };
}
/**
 * @template T
 * @param {?} __0
 * @return {?}
 */
function initFormControlFromTuple([value, validators]) {
    validators = validators || [];
    return initFormControlFromUpdate({ value, validators });
}
/**
 * @template T
 * @param {?} initialUpdate
 * @return {?}
 */
function initFormControlFromUpdate(initialUpdate) {
    return reduceFormControl({
        value: undefined,
        pristine: true,
        untouched: true,
        disabled: false,
        validators: [],
    }, initialUpdate);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 * @param {?} control
 * @return {?}
 */
function getFormControlErrors(control) {
    return control.validators.map((/**
     * @param {?} validator
     * @return {?}
     */
    validator => validator(control)));
}
/**
 * @template TControls
 * @param {?} summaries
 * @return {?}
 */
function getFormGroupErrors(summaries) {
    /** @type {?} */
    const errors = mapFormGroupControlSummaries(summaries, (/**
     * @param {?} summary
     * @return {?}
     */
    summary => summary.errors));
    Object.entries(errors)
        .filter((/**
     * @param {?} __0
     * @return {?}
     */
    ([_, value]) => value === null))
        .forEach((/**
     * @param {?} __0
     * @return {?}
     */
    ([key, _]) => delete errors[key]));
    return Object.keys(errors).length ? errors : null;
}
/**
 * @template T
 * @param {?} summaries
 * @return {?}
 */
function getFormArrayErrors(summaries) {
    /** @type {?} */
    const errors = summaries.map((/**
     * @param {?} summary
     * @return {?}
     */
    summary => summary.errors));
    return errors.filter(Boolean).length ? errors : null;
}
/**
 * Creates a `FormControlSummary` from the given `FormControlState`.
 * It is possible to add additional errors.
 *
 * @template T
 * @param {?} control The `FormControlState` which is used to create the `FormControlSummary`.
 * @param {...?} additionalErrors An array of `FormControlErrors` which will be merged into the errors of the control.
 * @return {?}
 */
function getFormControlSummary(control, ...additionalErrors) {
    /** @type {?} */
    const errors = mergeFormControlErrors(...getFormControlErrors(control), ...additionalErrors);
    return Object.assign({}, control, { errors, valid: errors === null });
}
/**
 * @template TControls
 * @param {?} controls
 * @param {...?} additionalErrors
 * @return {?}
 */
function getFormGroupControlSummaries(controls, ...additionalErrors) {
    /** @type {?} */
    const additionalError = mergeFormGroupErrors(...additionalErrors);
    return mapFormGroupControlStates(controls, (/**
     * @param {?} control
     * @param {?} key
     * @return {?}
     */
    (control, key) => getFormControlSummary(control, additionalError ? additionalError[key] : null)));
}
/**
 * @template T
 * @param {?} controls
 * @param {...?} additionalErrors
 * @return {?}
 */
function getFormArrayControlSummaries(controls, ...additionalErrors) {
    /** @type {?} */
    const additionalError = mergeFormArrayErrors(...additionalErrors);
    return controls.map((/**
     * @param {?} control
     * @param {?} i
     * @return {?}
     */
    (control, i) => getFormControlSummary(control, additionalError ? additionalError[i] : null)));
}
/**
 * @template TControls
 * @param {?} group
 * @return {?}
 */
function getFormGroupPristine(group) {
    return Object.values(group.controls).every((/**
     * @param {?} control
     * @return {?}
     */
    control => control.pristine));
}
/**
 * @template T
 * @param {?} array
 * @return {?}
 */
function getFormArrayPristine(array) {
    return array.controls.every((/**
     * @param {?} control
     * @return {?}
     */
    control => control.pristine));
}
/**
 * @template TControls
 * @param {?} group
 * @return {?}
 */
function getFormGroupUntouched(group) {
    return Object.values(group.controls).every((/**
     * @param {?} control
     * @return {?}
     */
    control => control.untouched));
}
/**
 * @template T
 * @param {?} array
 * @return {?}
 */
function getFormArrayUntouched(array) {
    return array.controls.every((/**
     * @param {?} control
     * @return {?}
     */
    control => control.untouched));
}
/**
 * @template T
 * @param {?} array
 * @return {?}
 */
function getFormArrayKeys(array) {
    return array.controls.map((/**
     * @param {?} _
     * @param {?} i
     * @return {?}
     */
    (_, i) => i));
}
/**
 * Creates a `FormGroupSummary` from the given `FormGroupState`.
 * It is possible to add additional errors.
 *
 * @template TControls
 * @param {?} group The input `FormGroupState`. Used to create the `FormGroupSummary`.
 * @param {...?} additionalErrors An array of additional `FormGroupErrors`, which will be merged into the errors of the group and of each control.
 * @return {?}
 */
function getFormGroupSummary(group, ...additionalErrors) {
    /** @type {?} */
    const summaries = getFormGroupControlSummaries(group.controls, ...additionalErrors);
    /** @type {?} */
    const errors = getFormGroupErrors(summaries);
    return {
        controls: summaries,
        pristine: getFormGroupPristine(group),
        untouched: getFormGroupUntouched(group),
        errors,
        valid: errors === null,
    };
}
/**
 * Creates a `FormArraySummary` from the given `FormArrayState`.
 * It is possible to add additional errors.
 *
 * @template T
 * @param {?} array The input `FormArrayState`. Used to create the `FormArraySummary`.
 * @param {...?} additionalErrors An array of additional `FormArrayErrors`, which will be merged into the errors of the array and of each control.
 * @return {?}
 */
function getFormArraySummary(array, ...additionalErrors) {
    /** @type {?} */
    const summaries = getFormArrayControlSummaries(array.controls, ...additionalErrors);
    /** @type {?} */
    const errors = getFormArrayErrors(summaries);
    return {
        controls: summaries,
        keys: getFormArrayKeys(array),
        pristine: getFormArrayPristine(array),
        untouched: getFormArrayUntouched(array),
        errors,
        valid: errors === null,
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function FormControls() { }
/**
 * Config from NgrxCleanForms.
 * For details of the different attributes view the TSDoc of specific attributes.
 * @record
 */
function FormsConfig() { }
if (false) {
    /**
     * Number of how many milliseconds needs to pass between individual updates of a `FormControl`.
     * There is no delay on the first update and also always an update after the delay.
     * @type {?}
     */
    FormsConfig.prototype.throttleTime;
    /**
     * Specifies whether only distinct values should be written to a `FormControl`.
     * Only applies to custom inputs added with `ControlValueAccessor`. Prevents update loops and uses `JSON.stringfy()` for comparison.
     * @type {?}
     */
    FormsConfig.prototype.distinctWritesOnly;
}
/**
 * @record
 * @template TControls
 */
function FormGroupBase() { }
/**
 * @record
 * @template T
 */
function FormArrayBase() { }
/**
 * @record
 * @template T
 */
function FormControlUpdate() { }
/**
 * @record
 * @template TControls
 */
function FormGroupUpdate() { }
if (false) {
    /** @type {?|undefined} */
    FormGroupUpdate.prototype.controls;
}
/**
 * @record
 * @template T
 */
function FormArrayUpdate() { }
if (false) {
    /** @type {?|undefined} */
    FormArrayUpdate.prototype.controls;
}
/**
 * Explicit type to create a new FormControl.
 *
 * Identical to FormControlUpdate, except for `value` which is a required attribute here.
 * @record
 * @template T
 */
function FormControlInitUpdate() { }
if (false) {
    /**
     * Initial value of the control.
     * @type {?}
     */
    FormControlInitUpdate.prototype.value;
}
/**
 * @record
 */
function FormControlErrors() { }
/**
 * Represents the state of a single FormControl.
 * @record
 * @template T
 */
function FormControlState() { }
if (false) {
    /** @type {?} */
    FormControlState.prototype.value;
    /**
     * Indicates whether the value was not yet changed.
     * @type {?}
     */
    FormControlState.prototype.pristine;
    /**
     * Indicates whether the FormControl was not yet visited.
     * @type {?}
     */
    FormControlState.prototype.untouched;
    /**
     * Indicates whether the FormControl is disabled.
     * @type {?}
     */
    FormControlState.prototype.disabled;
    /**
     * Validators which will be used to calculate the errors.
     * Mainly used inside the summary creation.
     * @type {?}
     */
    FormControlState.prototype.validators;
}
/**
 * A FormGroup can contain multiple FormControls of different types, associated to a key.
 * @record
 * @template TControls
 */
function FormGroupState() { }
if (false) {
    /**
     * An object containing all controls of this group.
     * @type {?}
     */
    FormGroupState.prototype.controls;
}
/**
 * A FormArray can contain any number of FormControls.
 * @record
 * @template T
 */
function FormArrayState() { }
if (false) {
    /**
     * An array of all controls contained by this form.
     * @type {?}
     */
    FormArrayState.prototype.controls;
}
/**
 * A summary of a `FormControlState`. Contains additional information.
 * @record
 * @template T
 */
function FormControlSummary() { }
if (false) {
    /**
     * An object containing all errors.
     *
     * Null if no errors were found.
     * @type {?}
     */
    FormControlSummary.prototype.errors;
    /**
     * Whether the FormControl has any errors.
     * @type {?}
     */
    FormControlSummary.prototype.valid;
}
/**
 * A summary of a `FormGroupState`. Contains additional information.
 * @record
 * @template TControls
 */
function FormGroupSummary() { }
if (false) {
    /**
     * An object containing summaries of all controls.
     * @type {?}
     */
    FormGroupSummary.prototype.controls;
    /**
     * An object containing all errors. The used keys are the same as for the controls.
     *
     * If no error is found this value is null.
     * @type {?}
     */
    FormGroupSummary.prototype.errors;
    /**
     * Indicates whether all of the FormControlSummaries inside this group are valid.
     * @type {?}
     */
    FormGroupSummary.prototype.valid;
    /**
     * Indicates whether all of the values inside this group were not yet changed.
     * @type {?}
     */
    FormGroupSummary.prototype.pristine;
    /**
     * Indicates whether all FormControls inside this group were not yet visited.
     * @type {?}
     */
    FormGroupSummary.prototype.untouched;
}
/**
 * A summary of a `FormArrayState`. Contains additional information.
 * @record
 * @template T
 */
function FormArraySummary() { }
if (false) {
    /** @type {?} */
    FormArraySummary.prototype.controls;
    /**
     * A convenience number array which contains the keys of the controls.
     *
     * For a array with the length of 2 this would be: `[0, 1]`
     * @type {?}
     */
    FormArraySummary.prototype.keys;
    /**
     * An array containing all
     * @type {?}
     */
    FormArraySummary.prototype.errors;
    /**
     * Indicates whether all of the FormControlSummaries inside this array are valid.
     * @type {?}
     */
    FormArraySummary.prototype.valid;
    /**
     * Indicates whether all of the values inside this array were not yet changed.
     * @type {?}
     */
    FormArraySummary.prototype.pristine;
    /**
     * Indicates whether all FormControls inside this array were not yet visited.
     * @type {?}
     */
    FormArraySummary.prototype.untouched;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const CONFIG_TOKEN = new InjectionToken('NGRX_CLEAN_FORMS_CONFIG');
/** @type {?} */
const defaultConfig = {
    throttleTime: 20,
    distinctWritesOnly: true,
};
/** @type {?} */
const throttle = (/**
 * @param {?} config
 * @return {?}
 */
(config) => throttleTime(config.throttleTime, asyncScheduler, {
    leading: true,
    trailing: true,
}));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const cssClasses = {
    invalid: 'ng-invalid',
    valid: 'ng-valid',
    pristine: 'ng-pristine',
    dirty: 'ng-dirty',
    touched: 'ng-touched',
    untouched: 'ng-untouched',
};
/** @type {?} */
const CONTROL_DIRECTIVE_SELECTOR = `ngrxFormControl`;
/**
 * @abstract
 * @template T
 */
class AbstractControlDirective {
    /**
     * @param {?} ref
     * @param {?} r2
     * @param {?} injectedConfig
     */
    constructor(ref, r2, injectedConfig) {
        this.ref = ref;
        this.r2 = r2;
        this.injectedConfig = injectedConfig;
        this.controlUpdate = new EventEmitter(true);
        this.touched$ = new Subject();
        this.value$ = new Subject();
        this.config$ = new BehaviorSubject(this.injectedConfig);
        this.summarySubscription = new Subscription();
        this.config$
            .pipe(switchMap((/**
         * @param {?} config
         * @return {?}
         */
        config => merge(this.touched$.pipe(throttle(config)), this.value$.pipe(throttle(config))))))
            .subscribe((/**
         * @param {?} value
         * @return {?}
         */
        value => this.controlUpdate.emit(value)));
    }
    /**
     * @param {?} controlSummary
     * @return {?}
     */
    set setControlSummary(controlSummary) {
        if (!controlSummary) {
            return;
        }
        this.updateSummary(controlSummary);
    }
    /**
     * @param {?} inputConfig
     * @return {?}
     */
    set setConfig(inputConfig) {
        /** @type {?} */
        const config = Object.assign({}, this.injectedConfig, inputConfig);
        this.config$.next(config);
    }
    /**
     * @return {?}
     */
    emitTouched() {
        this.touched$.next({ untouched: false });
    }
    /**
     * @param {?} value
     * @return {?}
     */
    emitValue(value) {
        this.value$.next({ value, pristine: false });
    }
    /**
     * @param {?} summary
     * @return {?}
     */
    updateSummary(summary) {
        this.setValue(summary.value);
        this.setDisabled(summary.disabled);
        this.chooseClass(cssClasses.invalid, cssClasses.valid, summary.valid);
        this.chooseClass(cssClasses.dirty, cssClasses.pristine, summary.pristine);
        this.chooseClass(cssClasses.touched, cssClasses.untouched, summary.untouched);
    }
    /**
     * @param {?} class1
     * @param {?} class2
     * @param {?} chooseSecond
     * @return {?}
     */
    chooseClass(class1, class2, chooseSecond) {
        if (chooseSecond) {
            this.r2.addClass(this.ref.nativeElement, class2);
            this.r2.removeClass(this.ref.nativeElement, class1);
        }
        else {
            this.r2.addClass(this.ref.nativeElement, class1);
            this.r2.removeClass(this.ref.nativeElement, class2);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.summarySubscription.unsubscribe();
        this.config$.complete();
        this.touched$.complete();
        this.value$.complete();
    }
}
/** @nocollapse */
AbstractControlDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
];
AbstractControlDirective.propDecorators = {
    controlKey: [{ type: Input, args: [CONTROL_DIRECTIVE_SELECTOR,] }],
    setControlSummary: [{ type: Input, args: ['controlSummary',] }],
    setConfig: [{ type: Input, args: ['controlConfig',] }],
    controlUpdate: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    AbstractControlDirective.prototype.controlKey;
    /** @type {?} */
    AbstractControlDirective.prototype.controlUpdate;
    /**
     * @type {?}
     * @private
     */
    AbstractControlDirective.prototype.touched$;
    /**
     * @type {?}
     * @private
     */
    AbstractControlDirective.prototype.value$;
    /**
     * @type {?}
     * @protected
     */
    AbstractControlDirective.prototype.config$;
    /**
     * @type {?}
     * @private
     */
    AbstractControlDirective.prototype.summarySubscription;
    /**
     * @type {?}
     * @protected
     */
    AbstractControlDirective.prototype.ref;
    /**
     * @type {?}
     * @protected
     */
    AbstractControlDirective.prototype.r2;
    /**
     * @type {?}
     * @private
     */
    AbstractControlDirective.prototype.injectedConfig;
    /**
     * @abstract
     * @param {?} value
     * @return {?}
     */
    AbstractControlDirective.prototype.setValue = function (value) { };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RangeInputControlDirective extends AbstractControlDirective {
    /**
     * @param {?} ref
     * @param {?} r2
     * @param {?} config
     */
    constructor(ref, r2, config) {
        super(ref, r2, config);
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    onInput($event) {
        /** @type {?} */
        const value = ((/** @type {?} */ ($event.target))).value;
        this.emitValue(value === '' ? null : parseFloat(value));
    }
    /**
     * @return {?}
     */
    onBlur() {
        this.emitTouched();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setValue(value) {
        this.r2.setProperty(this.ref.nativeElement, 'value', value);
    }
    /**
     * @param {?} disabled
     * @return {?}
     */
    setDisabled(disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    }
}
RangeInputControlDirective.decorators = [
    { type: Directive, args: [{
                selector: `input[type="range"][${CONTROL_DIRECTIVE_SELECTOR}]`,
            },] }
];
/** @nocollapse */
RangeInputControlDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
];
RangeInputControlDirective.propDecorators = {
    onInput: [{ type: HostListener, args: ['input', ['$event'],] }],
    onBlur: [{ type: HostListener, args: ['blur',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const RadioControlNotSupported = Error(
// tslint:disable-next-line: max-line-length
`Radio inputs are not yet supported. If you need to use one consider wrapping it in a Component which implements the ControlValueAccessor interface.`);
class RadioInputControlDirective extends AbstractControlDirective {
    /**
     * @param {?} ref
     * @param {?} r2
     * @param {?} config
     */
    constructor(ref, r2, config) {
        super(ref, r2, config);
        throw RadioControlNotSupported;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setValue(value) {
        throw RadioControlNotSupported;
    }
}
RadioInputControlDirective.decorators = [
    { type: Directive, args: [{
                selector: `input[type="radio"][${CONTROL_DIRECTIVE_SELECTOR}]`,
            },] }
];
/** @nocollapse */
RadioInputControlDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const SelectControlNotSupported = Error(
// tslint:disable-next-line: max-line-length
`Select inputs are not yet supported. If you need to use one consider wrapping it in a Component which implements the ControlValueAccessor interface.`);
class SelectInputControlDirective extends AbstractControlDirective {
    /**
     * @param {?} ref
     * @param {?} r2
     * @param {?} config
     */
    constructor(ref, r2, config) {
        super(ref, r2, config);
        throw SelectControlNotSupported;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setValue(value) {
        throw SelectControlNotSupported;
    }
}
SelectInputControlDirective.decorators = [
    { type: Directive, args: [{
                selector: `select[${CONTROL_DIRECTIVE_SELECTOR}]`,
            },] }
];
/** @nocollapse */
SelectInputControlDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NumberInputControlDirective extends AbstractControlDirective {
    /**
     * @param {?} ref
     * @param {?} r2
     * @param {?} config
     */
    constructor(ref, r2, config) {
        super(ref, r2, config);
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    onInput($event) {
        /** @type {?} */
        const value = ((/** @type {?} */ ($event.target))).value;
        this.emitValue(value === '' ? null : parseFloat(value));
    }
    /**
     * @return {?}
     */
    onBlur() {
        this.emitTouched();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setValue(value) {
        this.r2.setProperty(this.ref.nativeElement, 'value', value === null ? '' : value);
    }
    /**
     * @param {?} disabled
     * @return {?}
     */
    setDisabled(disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    }
}
NumberInputControlDirective.decorators = [
    { type: Directive, args: [{
                selector: `input[type="number"][${CONTROL_DIRECTIVE_SELECTOR}]`,
            },] }
];
/** @nocollapse */
NumberInputControlDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
];
NumberInputControlDirective.propDecorators = {
    onInput: [{ type: HostListener, args: ['input', ['$event'],] }],
    onBlur: [{ type: HostListener, args: ['blur',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TextInputControlDirective extends AbstractControlDirective {
    /**
     * @param {?} ref
     * @param {?} r2
     * @param {?} config
     */
    constructor(ref, r2, config) {
        super(ref, r2, config);
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    onInput($event) {
        /** @type {?} */
        const value = ((/** @type {?} */ ($event.target))).value;
        this.emitValue(value);
    }
    /**
     * @return {?}
     */
    onBlur() {
        this.emitTouched();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setValue(value) {
        this.r2.setProperty(this.ref.nativeElement, 'value', value);
    }
    /**
     * @param {?} disabled
     * @return {?}
     */
    setDisabled(disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    }
}
TextInputControlDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line: max-line-length
                selector: `input[type="text"][${CONTROL_DIRECTIVE_SELECTOR}],textarea[${CONTROL_DIRECTIVE_SELECTOR}],input:not([type="checkbox"]):not([type="number"]):not([type="range"]):not([type="radio"])[${CONTROL_DIRECTIVE_SELECTOR}]`,
            },] }
];
/** @nocollapse */
TextInputControlDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
];
TextInputControlDirective.propDecorators = {
    onInput: [{ type: HostListener, args: ['input', ['$event'],] }],
    onBlur: [{ type: HostListener, args: ['blur',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ValueAccessorConnectorDirective extends AbstractControlDirective {
    /**
     * @param {?} ref
     * @param {?} r2
     * @param {?} config
     * @param {?} accessors
     */
    constructor(ref, r2, config, accessors) {
        super(ref, r2, config);
        this.initAccessor(accessors);
    }
    /**
     * @param {?} accessors
     * @return {?}
     */
    initAccessor(accessors) {
        if (!accessors || !accessors.length) {
            return;
        }
        this.accessor = accessors[0];
        this.accessor.registerOnChange((/**
         * @param {?} val
         * @return {?}
         */
        val => super.emitValue(val)));
        this.accessor.registerOnTouched((/**
         * @return {?}
         */
        () => super.emitTouched()));
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setValue(value) {
        if (!this.accessor) {
            return;
        }
        if (this.config$.value.distinctWritesOnly && this.equalsLastValue(value)) {
            return;
        }
        this.accessor.writeValue(value);
    }
    /**
     * @param {?} disabled
     * @return {?}
     */
    setDisabled(disabled) {
        if (this.accessor && this.accessor.setDisabledState) {
            this.accessor.setDisabledState(disabled);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    equalsLastValue(value) {
        try {
            /** @type {?} */
            const valueString = JSON.stringify(value);
            /** @type {?} */
            const isEqual = valueString === this.lastValue;
            this.lastValue = valueString;
            return isEqual;
        }
        catch (_a) {
            return false;
        }
    }
}
ValueAccessorConnectorDirective.decorators = [
    { type: Directive, args: [{
                selector: `[${CONTROL_DIRECTIVE_SELECTOR}]`,
            },] }
];
/** @nocollapse */
ValueAccessorConnectorDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] },
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [NG_VALUE_ACCESSOR,] }] }
];
if (false) {
    /** @type {?} */
    ValueAccessorConnectorDirective.prototype.accessor;
    /** @type {?} */
    ValueAccessorConnectorDirective.prototype.lastValue;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CheckboxInputControlDirective extends AbstractControlDirective {
    /**
     * @param {?} ref
     * @param {?} r2
     * @param {?} config
     */
    constructor(ref, r2, config) {
        super(ref, r2, config);
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    onInput($event) {
        /** @type {?} */
        const checked = ((/** @type {?} */ ($event.target))).checked;
        this.emitValue(checked);
    }
    /**
     * @return {?}
     */
    onBlur() {
        this.emitTouched();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setValue(value) {
        this.r2.setProperty(this.ref.nativeElement, 'checked', value);
    }
    /**
     * @param {?} disabled
     * @return {?}
     */
    setDisabled(disabled) {
        this.r2.setProperty(this.ref.nativeElement, 'disabled', disabled);
    }
}
CheckboxInputControlDirective.decorators = [
    { type: Directive, args: [{
                selector: `input[type="checkbox"][${CONTROL_DIRECTIVE_SELECTOR}]`,
            },] }
];
/** @nocollapse */
CheckboxInputControlDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: undefined, decorators: [{ type: Inject, args: [CONFIG_TOKEN,] }] }
];
CheckboxInputControlDirective.propDecorators = {
    onInput: [{ type: HostListener, args: ['input', ['$event'],] }],
    onBlur: [{ type: HostListener, args: ['blur',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const controlDirectives = [
    TextInputControlDirective,
    CheckboxInputControlDirective,
    NumberInputControlDirective,
    RadioInputControlDirective,
    RangeInputControlDirective,
    SelectInputControlDirective,
    ValueAccessorConnectorDirective,
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const config = { descendants: true };
/**
 * @abstract
 */
class ControlChildren {
    /**
     * @protected
     * @return {?}
     */
    getChildren() {
        /** @type {?} */
        const queries$ = [
            this.textInputs,
            this.numberInputs,
            this.radioInputs,
            this.rangeInputs,
            this.selectInputs,
            this.customInputs,
            this.checkboxInputs,
        ]
            .filter(Boolean)
            .map((/**
         * @param {?} query
         * @return {?}
         */
        query => {
            return this.convertToObservable(query);
        }));
        return combineLatest(queries$).pipe(map((/**
         * @param {?} queries
         * @return {?}
         */
        queries => queries.reduce((/**
         * @param {?} q1
         * @param {?} q2
         * @return {?}
         */
        (q1, q2) => [...q1, ...q2]), []))));
    }
    /**
     * @private
     * @param {?} query
     * @return {?}
     */
    convertToObservable(query) {
        return query.changes.pipe(map((/**
         * @return {?}
         */
        () => query)), startWith(query), map((/**
         * @param {?} list
         * @return {?}
         */
        list => list.toArray())));
    }
}
ControlChildren.propDecorators = {
    textInputs: [{ type: ContentChildren, args: [TextInputControlDirective, config,] }],
    checkboxInputs: [{ type: ContentChildren, args: [CheckboxInputControlDirective, config,] }],
    numberInputs: [{ type: ContentChildren, args: [NumberInputControlDirective, config,] }],
    radioInputs: [{ type: ContentChildren, args: [RadioInputControlDirective, config,] }],
    rangeInputs: [{ type: ContentChildren, args: [RangeInputControlDirective, config,] }],
    selectInputs: [{ type: ContentChildren, args: [SelectInputControlDirective, config,] }],
    customInputs: [{ type: ContentChildren, args: [ValueAccessorConnectorDirective, config,] }]
};
if (false) {
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.textInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.checkboxInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.numberInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.radioInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.rangeInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.selectInputs;
    /**
     * @type {?}
     * @private
     */
    ControlChildren.prototype.customInputs;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 * @template Summary, Update
 */
class AbstractFormDirective extends ControlChildren {
    constructor() {
        super(...arguments);
        this.formUpdate = new EventEmitter();
        this.formSummary$ = new ReplaySubject(1);
    }
    /**
     * @param {?} summary
     * @return {?}
     */
    set setFormSummary(summary) {
        if (!summary) {
            return;
        }
        this.formSummary$.next(summary);
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        /** @type {?} */
        const children$ = this.getChildren();
        children$
            .pipe(map((/**
         * @param {?} children
         * @return {?}
         */
        children => children.map((/**
         * @param {?} child
         * @return {?}
         */
        child => child.controlUpdate.pipe(map((/**
         * @param {?} update
         * @return {?}
         */
        (update) => [update, child.controlKey]))))))), switchMap((/**
         * @param {?} children
         * @return {?}
         */
        children => merge(...children))))
            .subscribe((/**
         * @param {?} __0
         * @return {?}
         */
        ([update, key]) => this.emitUpdate(update, key)));
        combineLatest(children$, this.formSummary$).subscribe((/**
         * @param {?} __0
         * @return {?}
         */
        ([children, summary]) => this.updateChildren(children, summary)));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.formSummary$.complete();
    }
    /**
     * @param {?} children
     * @param {?} summary
     * @return {?}
     */
    updateChildren(children, summary) {
        if (!children.length) {
            return;
        }
        children.forEach((/**
         * @param {?} control
         * @return {?}
         */
        control => {
            control.updateSummary(summary.controls[control.controlKey]);
        }));
    }
}
AbstractFormDirective.propDecorators = {
    setFormSummary: [{ type: Input, args: ['formSummary',] }],
    formUpdate: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    AbstractFormDirective.prototype.formUpdate;
    /** @type {?} */
    AbstractFormDirective.prototype.formSummary$;
    /**
     * @abstract
     * @param {?} update
     * @param {?} key
     * @return {?}
     */
    AbstractFormDirective.prototype.emitUpdate = function (update, key) { };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FormGroupDirective extends AbstractFormDirective {
    /**
     * @param {?} update
     * @param {?} key
     * @return {?}
     */
    emitUpdate(update, key) {
        this.formUpdate.next({ controls: { [key]: update } });
    }
}
FormGroupDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngrxFormGroup]',
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FormArrayDirective extends AbstractFormDirective {
    /**
     * @param {?} update
     * @param {?} key
     * @return {?}
     */
    emitUpdate(update, key) {
        /** @type {?} */
        const index = parseInt(key.toString(), 10);
        /** @type {?} */
        const controls = new Array(index + 1);
        controls[index] = update;
        this.formUpdate.next({ controls });
    }
}
FormArrayDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngrxFormArray]',
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
const ɵ0 = defaultConfig;
/**
 * This module represents the entry point of ngrx-clean-forms.
 * It provides all necessary directives for the usage inside templates.
 *
 * The `.withConfig()` method can be used to pass a config to the module.
 *
 * - [npm](https://www.npmjs.com/package/ngrx-clean-forms)
 * - [documentation](https://github.com/NiklasPor/ngrx-clean-forms)
 */
class NgrxCleanFormsModule {
    /**
     * Creates a new module with the given config. Missing values will be set with default values.
     *
     * \@example
     * NgrxCleanFormsModule.withConfig({
     *   throttleTime: 15,
     * }),
     *
     * @param {?} config A partial `FormsConfig` object, supplying config paramters.
     *
     * @return {?}
     */
    static withConfig(config) {
        return {
            ngModule: NgrxCleanFormsModule,
            providers: [
                {
                    provide: CONFIG_TOKEN,
                    useValue: (/** @type {?} */ (Object.assign({}, defaultConfig, config))),
                },
            ],
        };
    }
}
NgrxCleanFormsModule.decorators = [
    { type: NgModule, args: [{
                declarations: [FormGroupDirective, FormArrayDirective, ...controlDirectives],
                imports: [CommonModule],
                exports: [FormGroupDirective, FormArrayDirective, ...controlDirectives],
                providers: [
                    {
                        provide: CONFIG_TOKEN,
                        useValue: ɵ0,
                    },
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgrxCleanFormsModule, getFormArraySummary, getFormControlSummary, getFormGroupSummary, initFormArray, initFormControl, initFormGroup, mapFormGroupControlStates, mapFormGroupControlSummaries, mapFormGroupControlUpdates, reduceFormArray, reduceFormControl, reduceFormGroup, validatorOf, mapFormControls as ɵa, FormGroupDirective as ɵb, AbstractFormDirective as ɵc, ControlChildren as ɵd, TextInputControlDirective as ɵe, CONTROL_DIRECTIVE_SELECTOR as ɵf, AbstractControlDirective as ɵg, CONFIG_TOKEN as ɵh, defaultConfig as ɵi, CheckboxInputControlDirective as ɵj, NumberInputControlDirective as ɵk, RadioInputControlDirective as ɵl, RangeInputControlDirective as ɵm, SelectInputControlDirective as ɵn, ValueAccessorConnectorDirective as ɵo, FormArrayDirective as ɵp, controlDirectives as ɵq };
//# sourceMappingURL=ngrx-clean-forms.js.map
