import { FormControlErrors, FormControls, FormControlState, FormControlSummary, FormGroupControlStates, FormGroupControlSummaries, FormGroupErrors, FormGroupState, FormGroupSummary, FormArraySummary, FormArrayErrors, FormArrayControlSummaries, FormArrayState } from './types';
export declare function getFormControlErrors<T>(control: FormControlState<T>): FormControlErrors[];
export declare function getFormGroupErrors<TControls extends FormControls>(summaries: FormGroupControlSummaries<TControls>): FormGroupErrors<TControls> | null;
export declare function getFormArrayErrors<T>(summaries: FormArrayControlSummaries<T>): FormArrayErrors | null;
/**
 * Creates a `FormControlSummary` from the given `FormControlState`.
 * It is possible to add additional errors.
 *
 * @param control The `FormControlState` which is used to create the `FormControlSummary`.
 * @param additionalErrors An array of `FormControlErrors` which will be merged into the errors of the control.
 */
export declare function getFormControlSummary<T>(control: FormControlState<T>, ...additionalErrors: FormControlErrors[]): FormControlSummary<T>;
export declare function getFormGroupControlSummaries<TControls extends FormControls>(controls: FormGroupControlStates<TControls>, ...additionalErrors: FormGroupErrors<TControls>[]): FormGroupControlSummaries<TControls>;
export declare function getFormArrayControlSummaries<T>(controls: FormControlState<T>[], ...additionalErrors: FormArrayErrors[]): FormControlSummary<T>[];
export declare function getFormGroupPristine<TControls extends FormControls>(group: FormGroupState<TControls>): boolean;
export declare function getFormArrayPristine<T>(array: FormArrayState<T>): boolean;
export declare function getFormGroupUntouched<TControls extends FormControls>(group: FormGroupState<TControls>): boolean;
export declare function getFormArrayUntouched<T>(array: FormArrayState<T>): boolean;
export declare function getFormArrayKeys<T>(array: FormArrayState<T>): number[];
/**
 * Creates a `FormGroupSummary` from the given `FormGroupState`.
 * It is possible to add additional errors.
 *
 * @param group The input `FormGroupState`. Used to create the `FormGroupSummary`.
 * @param additionalErrors An array of additional `FormGroupErrors`, which will be merged into the errors of the group and of each control.
 */
export declare function getFormGroupSummary<TControls extends FormControls>(group: FormGroupState<TControls>, ...additionalErrors: FormGroupErrors<TControls>[]): FormGroupSummary<TControls>;
/**
 * Creates a `FormArraySummary` from the given `FormArrayState`.
 * It is possible to add additional errors.
 *
 * @param array The input `FormArrayState`. Used to create the `FormArraySummary`.
 * @param additionalErrors An array of additional `FormArrayErrors`, which will be merged into the errors of the array and of each control.
 */
export declare function getFormArraySummary<T>(array: FormArrayState<T>, ...additionalErrors: FormArrayErrors[]): FormArraySummary<T>;
